/*
 * File:     SearchDialog.java
 * Project:  MPI Linguistic Application
 * Date:     May 2014
 */

package ims.tiger.gui.tigergrapheditor;

import ims.tiger.corpus.Feature;
import ims.tiger.corpus.Header;
import ims.tiger.corpus.Sentence;
import ims.tiger.gui.tigergraphviewer.forest.ResultForest;

import ims.tiger.query.api.QueryIndexException;
import ims.tiger.system.Constants;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.text.SimpleDateFormat;

import java.util.Date;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import ims.tiger.query.internalapi.InternalCorpusQueryManager;
import org.apache.log4j.Logger;

/**
 * $Id: SearchDialog.java,v 1.1 2007/03/23 14:51:23 klasal Exp $
 *
 * @author $author$
 * @version $Revision: 1.1 $
 */
public class SearchDialog extends JDialog {
    /** Return value from class method if YES is chosen. */
    public static final int OK_OPTION = 0;

    /** Return value from class method if CANCEL is chosen. */
    public static final int CANCEL_OPTION = 2;

    /** Holds value of property DOCUMENT ME! */
    private final Header header;

    /** Holds value of property DOCUMENT ME! */
    private final JTextField wordField = new JTextField(25);

    /** Holds value of property DOCUMENT ME! */
    private final SimpleDateFormat dateFormat = new SimpleDateFormat(
            "yyyy-MM-dd'T'H:mm:ssz");

    /** Holds value of property DOCUMENT ME! */
    private final String defaultFileFormat = "NeGra format, version 3";

    /** stores value of option */
    public int option = CANCEL_OPTION;
    
    /** stores searched value */
    private static String searchval = "";
    
    /** stores value of the dropdown menu for regex/caps*/
    private static String dropval = "";
    
    /** stores value of dropdown menu for word/pos/lemma */
    private static String searchCat = "";
    
    private static Boolean startFromBeginning = false;
    
    private static DefaultComboBoxModel dropdown = new DefaultComboBoxModel();

    private static JComboBox comboBox = new JComboBox(dropdown);

    private static DefaultComboBoxModel catBox = new DefaultComboBoxModel();

    private static JComboBox catComboBox = new JComboBox(catBox);
    
    private static JCheckBox wrapAround = new JCheckBox("start from beginning");
    

    /**
     * Creates a new NewFeatureFileDialog object.
     *
     * @param parent parent frame
     * @param header header to be reset
     */
    private SearchDialog(JFrame parent, Header header) {
        super(parent, true);
        this.header = header;
        makeLayout();
        pack();
    }

    /**
     * shows dialog and returns option (OK or CANCEL)
     *
     * @param parent parent frame
     * @param header header to be reset
     *
     * @return OK_OPTION or CANCEL_OPTION
     */
    public static SearchDialog showSearchDialog(JFrame parent, Header header) {
        SearchDialog dialog = new SearchDialog(parent, header);
        dialog.show();

        return dialog;
    }

    public static String getSearchVal(){
    	return searchval;
    }
    
    public static String getDropdown(){
    	return dropval;
    }
    
    public static String getSearchCat(){
    	return searchCat;
    }
    
    public static Boolean getStartFromBeginning(){
    	return startFromBeginning;
    }
    
    private void close() {
        setVisible(false);
        dispose();
    }

    private void makeLayout() {
        setTitle("Search");
        getContentPane().setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.insets = new Insets(10, 10, 10, 10);

        JLabel label = new JLabel("Search");
        label.setHorizontalAlignment(JLabel.CENTER);
        label.setFont(label.getFont().deriveFont(16f));
        getContentPane().add(label, c);

        c.insets = new Insets(5, 5, 0, 5);
        label = new JLabel("Word:");
        getContentPane().add(label, c);

        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);
        wordField.setBorder(new LineBorder(Color.GRAY));
        getContentPane().add(wordField, c);
        wordField.setText(searchval);
        
        JPanel panel = new JPanel();
        if (dropdown.getSize() == 0){
	        dropdown.addElement("ignore case");
	        dropdown.addElement("enforce case");
	        dropdown.addElement("regex");
        }
        panel.add(comboBox);
        getContentPane().add(panel);
        
        if (catBox.getSize() == 0){
	        catBox.addElement("word");
	        catBox.addElement("pos");
	        catBox.addElement("lemma");
        }
        panel.add(catComboBox);
        getContentPane().add(panel);
        
        panel.add(wrapAround);

        JButton button = new JButton("OK");
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                	SearchDialog.searchval = wordField.getText();
                	SearchDialog.dropval = (String) comboBox.getSelectedItem();
                	SearchDialog.searchCat = (String) catComboBox.getSelectedItem();
                	SearchDialog.startFromBeginning = wrapAround.isSelected();
                    option = OK_OPTION;
                    close();
                }
            });

        JPanel buttonPanel = new JPanel(new GridLayout(1, 0, 5, 5));
        buttonPanel.add(button);
        button = new JButton("Cancel");
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    close();
                }
            });
        buttonPanel.add(button);
        c.weighty = 0;
        c.fill = GridBagConstraints.NONE;
        getContentPane().add(buttonPanel, c);

    }
}
