/*
 * File:     TerminalNodeEditor.java
 * Project:  MPI Linguistic Application
 * Date:     07 February 2007
 *
 * Copyright (C) 2001-2007  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ims.tiger.gui.tigergraphviewer.draw;

import ims.tiger.corpus.Feature;
import ims.tiger.corpus.Header;
import ims.tiger.corpus.Sentence;
import ims.tiger.corpus.T_Node;
import ims.tiger.gui.tigergraphviewer.forest.CorpusForest;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import se.gu.spraakbanken.saldo.SaldoCompleter;


/**
 * DOCUMENT ME!
 * $Id: jalopy_gnu_src_dist.xml,v 1.3 2007/02/06 13:30:33 hasloe Exp $
 * @author $Author: hasloe $
 * @version $Revision: 1.3 $
 */
public class TerminalNodeEditor extends JDialog {
    /** Holds value of property DOCUMENT ME! */
    private /* final */ T_Node node;

    /** Holds value of property DOCUMENT ME! */
    private final String[] featureNames;
    private JComponent[] inputComponents;

    private GraphPanelEditorMouseListener listener;
    private JComboBox posComponent, msdComponent;
    private Header header;
    
    private void writeBackFeatures(GraphPanel graphPanel, CorpusForest forest) {
        for (int i = 0; i < inputComponents.length; i++) {
            TerminalNodeEditor.this.node.setFeature(featureNames[i],
                    TerminalNodeEditor.this.getValue(i));
        }
        graphPanel.setSentence(graphPanel.getCurrentDisplaySentence());
        graphPanel.repaint();
        forest.setModified(true);
    }

    // TODO move the selection management stuff from the mouse listener into a
    // separate class.
    
    private void goToNext(GraphPanel graphPanel, CorpusForest forest) {
        T_Node n = getNode();
        String id = n.getID(); 
        DisplaySentence ds = graphPanel.getCurrentDisplaySentence();
        Sentence s = ds.getSentence();
        int position = s.getTerminalPositionOf(id);
        if(position < s.getTerminalsSize() - 1) {
            T_Node next = (T_Node) s.getTerminalAt(position + 1);
            
            ds.getDisplayLeafNode(position).setHighlightedNode(false);
            DisplayNode dn = ds.getDisplayLeafNode(position + 1);

            listener.reset();
            dn.setHighlightedNode(true);
            graphPanel.setHighlightedNode(dn);
            listener.addToClickedNodes(position + 1);
            listener.updateActions();
            
            this.node = next;
            graphPanel.repaint();
        }
    }

    private void setFeatureValues() {
        for (int i = 0; i < featureNames.length; i++) {
            Feature f = header.getTFeature(featureNames[i]);
            List features = f.getItems();
            if (features.size() > 0) {
                JComboBox comboBox = (JComboBox) inputComponents[i];
                // TODO
                comboBox.setSelectedItem(node.getFeature(featureNames[i]));
            } else {
                JTextField textField = (JTextField) inputComponents[i];
                textField.setText(node.getFeature(featureNames[i]));
            }            
        }
    }

    private static final String MSD_FEATURE_NAME = "msd";
    private static final String POS_FEATURE_NAME = "pos";
    
    private void setMSDValues(String pos) {
        msdComponent.removeAllItems();
        Feature f = header.getTFeature(MSD_FEATURE_NAME);
        for(Object item: f.getItems())
            if( ((String) item).startsWith(pos) ) {
                msdComponent.addItem(item);
            }
    }
    
    private void initializeFeatureComponents(Header header, KeyListener keyListener,
            GridBagConstraints c, JPanel panel) {

        //JComponent toSelect = null;
        String initPos = node.getFeature(POS_FEATURE_NAME);
        
        for (int i = 0; i < featureNames.length; i++) {

            Feature f = header.getTFeature(featureNames[i]);
            List features = f.getItems();

            if (features.size() > 0) {

                JComboBox comboBox;
                if(featureNames[i].equals(MSD_FEATURE_NAME)) {
                    comboBox = new JComboBox();
                    msdComponent = comboBox;
                    setMSDValues(initPos);
                } else {
                    comboBox = new JComboBox(features.toArray());
                    ComboboxToolTipRenderer renderer = new ComboboxToolTipRenderer(features, f);
                    comboBox.setRenderer(renderer);
                }
                
                if(featureNames[i].equals(POS_FEATURE_NAME))
                    posComponent = comboBox;

                /* RJ hack! */
                if(/*featureNames[i].equals("pos")*/ true) {
                    comboBox.setEditable(false);
                    /*if(toSelect == null)
                        toSelect = comboBox;*/
                } /*else
                    comboBox.setEditable(true);*/
                /* End hack. */
                comboBox.addKeyListener(keyListener);
                // TODO
                comboBox.setSelectedItem(node.getFeature(featureNames[i]));
                //String ft = featureNames[i] + ": " + feature
                comboBox.setFont(getFont());
                
                inputComponents[i] = comboBox;
            } else {
                JTextField textField = new JTextField(15);
                textField.setPreferredSize(new Dimension(
                        textField.getPreferredSize().width,
                        (new JComboBox()).getPreferredSize().height));
                textField.setText(node.getFeature(featureNames[i]));
                inputComponents[i] = textField;

                /* RJ hack! */
                if(featureNames[i].equals("word"))
                    textField.setEditable(false);
                else if(featureNames[i].equals("saldo")) {
                    textField.setEditable(true);
                    textField.addKeyListener(new SaldoCompleter(textField));
                } else
                    textField.setEditable(true);
                /* End hack. */

                textField.addKeyListener(keyListener);

            }            
            
            c.gridwidth = GridBagConstraints.RELATIVE;
            c.anchor = GridBagConstraints.EAST;
            c.fill = GridBagConstraints.NONE;
            panel.add(new JLabel(featureNames[i]), c);
            c.anchor = GridBagConstraints.WEST;
            c.gridwidth = GridBagConstraints.REMAINDER;
            c.fill = GridBagConstraints.BOTH;
            panel.add(inputComponents[i], c);

        }
        
        posComponent.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setMSDValues((String) posComponent.getSelectedItem());
            }
        });
    }
    
    /**
     * Creates a new TerminalNodeEditor instance
     *
     * @param node DOCUMENT ME!
     * @param header DOCUMENT ME!
     * @param graphPanel DOCUMENT ME!
     * @param forest DOCUMENT ME!
     * @param graphPanelEditorMouseListener 
     */
    public TerminalNodeEditor(T_Node node, final Header header,
        final GraphPanel graphPanel, final CorpusForest forest,
        GraphPanelEditorMouseListener listener) {
        this.node = node;
        this.listener = listener;
        this.header = header;
        
        featureNames = (String[]) header.getAllTFeatureNames().toArray(new String[0]);

        Action okAction = new AbstractAction("OK") {
                public void actionPerformed(ActionEvent e) {
                    writeBackFeatures(graphPanel, forest);
                    dispose();
                }
            };

        Action cancelAction = new AbstractAction("Cancel") {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            };
            
        Action nextAction = new AbstractAction("Next") {
                public void actionPerformed(ActionEvent e) {
                    writeBackFeatures(graphPanel, forest);                    
                    goToNext(graphPanel, forest);
                    setFeatureValues();
                }
            };

        setTitle("Edit node");
        getContentPane().setLayout(new BorderLayout());

        JLabel headLabel = new JLabel("Terminal node features");
        headLabel.setHorizontalAlignment(JLabel.CENTER);
        headLabel.setBorder(new javax.swing.border.EmptyBorder(5, 5, 5, 5));
        getContentPane().add(headLabel, BorderLayout.NORTH);
        inputComponents = new JComponent[featureNames.length];

        JPanel panel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(5, 5, 5, 5);

        KeyListener keyListener = new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER) {
                    /*for (int j = 0; j < inputComponents.length; j++) {
                    TerminalNodeEditor.this.node.setFeature(featureNames[j],
                        TerminalNodeEditor.this.getValue(j));
                }
                graphPanel.setSentence(graphPanel.getCurrentDisplaySentence());
                graphPanel.repaint();
                forest.setModified(true);
                dispose();*/
                
                    writeBackFeatures(graphPanel, forest);                    
                    goToNext(graphPanel, forest);
                    setFeatureValues();
                }
            }
        };
        
        initializeFeatureComponents(header, keyListener, c, panel);

        if(posComponent != null) {
            SwingUtilities.invokeLater( new Runnable() { 
                public void run() { 
                    posComponent.requestFocusInWindow(); 
                }
            } );
        }
        
//      JComponent toSelect = null;
//      
//      for (int i = 0; i < featureNames.length; i++) {
//          //System.out.println("Feature name: " + featureNames[i]);
//          
//          Feature f = header.getTFeature(featureNames[i]);
//          //List features = header.getTFeature(featureNames[i]).getItems();
//          List features = f.getItems();
//
//          if (features.size() > 0) {
//              JComboBox comboBox = new JComboBox(features.toArray());
//              ComboboxToolTipRenderer renderer = new ComboboxToolTipRenderer(features, f);
//              comboBox.setRenderer(renderer);
//
//              //comboBox.setEditable(true);
//
//              /* RJ hack! */
//              if(/*featureNames[i].equals("pos")*/ true) {
//                  comboBox.setEditable(false);
//                  if(toSelect == null)
//                      toSelect = comboBox;
//              } /*else
//                  comboBox.setEditable(true);*/
//              /* End hack. */
//              
//              comboBox.addKeyListener(keyListener);
//              
//              // TODO
//              comboBox.setSelectedItem(node.getFeature(featureNames[i]));
//              
//              
//              
//              //String ft = featureNames[i] + ": " + feature
//                
//                comboBox.setFont(getFont());
//                inputComponents[i] = comboBox;
//            } else {
//                JTextField textField = new JTextField(15);
//                textField.setPreferredSize(new Dimension(
//                        textField.getPreferredSize().width,
//                        (new JComboBox()).getPreferredSize().height));
//                textField.setText(node.getFeature(featureNames[i]));
//                inputComponents[i] = textField;
//                
//                /* RJ hack! */
//                if(featureNames[i].equals("word"))
//                    textField.setEditable(false);
//                else if(featureNames[i].equals("saldo")) {
//                    textField.setEditable(true);
//                    textField.addKeyListener(new SaldoCompleter(textField));
//                } else
//
//                    textField.setEditable(true);
//                /* End hack. */
//                
//                textField.addKeyListener(keyListener);
//                
//            }
//
//            c.gridwidth = GridBagConstraints.RELATIVE;
//            c.anchor = GridBagConstraints.EAST;
//            c.fill = GridBagConstraints.NONE;
//            panel.add(new JLabel(featureNames[i]), c);
//            c.anchor = GridBagConstraints.WEST;
//            c.gridwidth = GridBagConstraints.REMAINDER;
//            c.fill = GridBagConstraints.BOTH;
//            panel.add(inputComponents[i], c);
//                        
//            if(false && toSelect != null) {
//                final Component fc = toSelect;
//                SwingUtilities.invokeLater( new Runnable() { 
//                    public void run() { 
//                            fc.requestFocusInWindow(); 
//                        }
//                    } );
//            }
//            
//        }

        getContentPane().add(panel, BorderLayout.CENTER);

        panel = new JPanel();

        JPanel insetPanel = new JPanel(new GridLayout(1, 0));
        insetPanel.add(new JButton(okAction));
        insetPanel.add(new JButton(cancelAction));
        insetPanel.add(new JButton(nextAction));
        panel.add(insetPanel);
        getContentPane().add(panel, BorderLayout.SOUTH);
        
        pack();
        setLocationRelativeTo(getParent());
    }

    private T_Node getNode() {
        return node;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getValue(int i) {
        if (inputComponents[i] instanceof JTextField) {
            return ((JTextField) inputComponents[i]).getText();
        }

        return (String) ((JComboBox) inputComponents[i]).getSelectedItem();
    }
    
    
    /* RJ */
    static class ComboboxToolTipRenderer extends DefaultListCellRenderer {
        ArrayList tooltips;
        
        ComboboxToolTipRenderer(List features, Feature f) {
            tooltips = new ArrayList();
            //for(Object o: features) {
	    for(int i = 0; i < features.size(); i++) {		
                String tt = f.getDescription((String) features.get(i));
                if(tt == null)
                    throw new RuntimeException("?");
                tooltips.add(tt);
            }
        }

        public Component getListCellRendererComponent(JList list, Object value,
                            int index, boolean isSelected, boolean cellHasFocus) {

            JComponent comp = (JComponent) super.getListCellRendererComponent(list,
                    value, index, isSelected, cellHasFocus);

            if (-1 < index && null != value && null != tooltips) {
		list.setToolTipText((String)tooltips.get(index));
	    }
            return comp;
        }

    }

    
}
