package se.gu.spraakbanken.saldo;

import java.io.*;
import java.util.*;
import java.util.zip.GZIPInputStream;

import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class SimpleSaldoLexicon {

    private String[] ids;
    
    public SimpleSaldoLexicon(String filename) throws IOException {
        try {
            System.err.print("Reading dictionary..."); System.err.flush();
            XMLReader reader = makeXMLReader();
            SaldoParserCallback spc = new SaldoParserCallback();
            reader.setContentHandler(spc);
            
            InputStream fis = new FileInputStream(filename);            
            if(filename.endsWith(".gz"))
                fis = new GZIPInputStream(fis);
            
            reader.parse(new InputSource(fis));
            System.err.println(" Done.");
            ids = (String[]) spc.entries.toArray(new String[spc.entries.size()]);            
            Arrays.sort(ids, String.CASE_INSENSITIVE_ORDER);
        } catch(Exception e) {
            if(e instanceof IOException)
                throw (IOException) e;
            throw new IOException(e);
        }
    }

    private final class SaldoParserCallback extends DefaultHandler 
    implements ContentHandler {
        private HashSet entries = new HashSet();    
        public void startElement(String namespace, String localname,  
                String type, Attributes attributes) {
            if(type.equals("Sense"))
                entries.add(attributes.getValue("id"));
        }
        
        public void endElement(String namespace, String localname,  
                String type) {
        } 
        
        public void characters(char[] ch, int start, int len) {
        }
        
    }
    
    private XMLReader makeXMLReader() throws Exception { 
        SAXParserFactory saxParserFactory   =  
            SAXParserFactory.newInstance(); 

        //saxParserFactory.setValidating(false);
        
        final SAXParser saxParser 
        = saxParserFactory.newSAXParser(); 

        final org.xml.sax.XMLReader parser    
        = saxParser.getXMLReader(); 
        
        try {
            parser.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd",
                    false);
        } 
        catch (Exception e) {
            System.err.println("could not set parser feature");
        }

        return parser; 
    }
 
    public static final class Index {
        int p;
        String s;
        public Index(int p, String s) {
            this.p = p;
            this.s = s;
        }
    }
    
    public Index byPrefix(String pfx) {
        int ix = Arrays.binarySearch(ids, pfx, String.CASE_INSENSITIVE_ORDER);
        if(ix < 0)
            ix = -ix - 1;
        if(ix >= ids.length)
            return null;
        if(ids[ix].length() < pfx.length())
            return null;
        String pfx2 = ids[ix].substring(0, pfx.length());
        if(pfx.equalsIgnoreCase(pfx2))
            return new Index(ix, ids[ix]);
        return null;
    }

    public Index previous(Index ix) {
        if(ix == null || ix.p <= 0)
            return null;
        else
            return new Index(ix.p-1, ids[ix.p-1]);
    }
    
    public Index next(Index ix) {
        if(ix == null || ix.p >= ids.length-1)
            return null;
        else
            return new Index(ix.p+1, ids[ix.p+1]);
    }
    
    
}
